@@ -1,6 +1,6 @@
 //===----------------------------------------------------------------------===//
 //
-// Copyright (c) 2012, 2013, 2014, 2015 The University of Utah
+// Copyright (c) 2012, 2013, 2014, 2015, 2016 The University of Utah
 // All rights reserved.
 //
 // This file is distributed under the University of Illinois Open Source
@@ -120,7 +120,7 @@ void ReplaceFunctionDefWithDecl::removeCtorInitializers(
   // namespace NS { struct A {}; }
   // struct B : NS::A { B() : NS::A() {} };
   SourceLocation Loc = RewriteHelper->getLocationFromLeftUntil(LocStart, ':');
-  Loc = RewriteHelper->getLocationFromLeftUntil(LocStart, ')');
+  Loc = RewriteHelper->getLocationFromLeftUntil(Loc, ')');
   TheRewriter.RemoveText(SourceRange(Loc.getLocWithOffset(1), 
                                      LocStart.getLocWithOffset(-1)));
   CXXConstructorDecl::init_const_iterator E = Ctor->init_end();
